phantom.onError = function(msg, trace) {
	var msgStack = ['PHANTOM ERROR: ' + msg];
	if (trace && trace.length) {
		msgStack.push('TRACE:');
		trace.forEach(function(t) {
			msgStack.push(' -> ' + (t.file || t.sourceURL) + ': ' + t.line + (t.function ? ' (in function ' + t.function + ')' : ''));
		});
	}
	console.error(msgStack.join('\n'));
	phantom.exit(1);
};

var inlineHeaderHeight = 0;
var inlineFooterHeight = 0;

if (system.args.length < 3) {
	console.log('Usage: printheaderfooter.js URL filename');
	phantom.exit(1);
} else {
	var address = system.args[1];
	var output = system.args[2];

	page.open(address, function (status) {
		if (status !== 'success') {
			console.log('Unable to load ' + address);
			phantom.exit();
		} else {
			initStatic();
			inlineHeaderHeight = page.evaluate(function(){
				var list = document.getElementsByTagName("pd4ml:page.header");
				if ( list && list[0] ) {
//					alert(list[0].offsetHeight);
					return list[0].offsetHeight;
				} else {
					return 0;
				}
			});
        
			inlineFooterHeight = page.evaluate(function(){
				var list = document.getElementsByTagName("pd4ml:page.footer");
				if ( list && list[0] ) {
//					alert(list[0].offsetHeight);
					return list[0].offsetHeight;
				} else {
					return 0;
				}
			});

			initDynamic(inlineHeaderHeight ? (inlineHeaderHeight + .5)*scale/96 : 0, inlineFooterHeight > 0 ? (inlineFooterHeight + .5)*scale/96 : 0);
			render();
		}
	});	
}

function initStatic() {
	page.settings.userAgent = 'PD4ML';
	page.viewportSize = { width: 1000, height: 1000 };
	page.zoomFactor = scale;

	page.onError = function (msg, trace) {
		console.log(msg);
		trace.forEach(function(item) {
			console.log('  ', item.file, ':', item.line);
		})
	};

	page.onAlert = function(msg) {
		console.log('ALERT: ' + msg);
	};

	page.evaluate(function () {
		var links = document.getElementsByTagName('link');
		for (var i = 0, len = links.length; i < len; ++i) {
			var link = links[i];
			if (link.rel == 'stylesheet') {
				if (link.media == 'screen') { link.media = ''; }
				if (link.media == 'print') { link.media = 'ignore'; }
			}
		}
	});

	page.evaluate(function() {
		var style = document.createElement("style");
		style.appendChild(document.createTextNode(""));
		document.head.appendChild(style);
		style.sheet.insertRule("pd4ml\\:page\\.header, pd4ml\\:page\\.footer {display: block; visibility: hidden; position: absolute}", 0);
		style.sheet.insertRule("pd4ml\\:page\\.break {display: block; page-break-before: always}", 0);
		style.sheet.insertRule("img {max-width: none !important}", 0);
		style.sheet.insertRule("a[href]:after { content:\"\"; text-decoration: none !important; }", 0);
	});

	if( page.injectJs(extraScript) ) {
		page.evaluate(function(base) {resolve(base)}, base);
	}

}

function render() {
	page.evaluate(function(baseUrl){
		var head = document.getElementsByTagName("head")[0];
		if ( baseUrl ) {
			if ( baseUrl.substring(0, 4) != 'http' && baseUrl.substring(0, 5) != 'file:' && baseUrl.substring(0, 1) != '#' ) {
				// baseUrl = "file:///" + baseUrl;
			}
			var base = document.getElementsByTagName("base")[0];
			if ( base ) {
				base.setAttribute("href", baseUrl);
			} else {
				var newBase = document.createElement("base");
				newBase.setAttribute("href", baseUrl);
				head.appendChild(newBase);
			}
		}
	}, base);

	window.setTimeout(function () {
		page.render(output);
		phantom.exit();
	}, 200);
}

function getInline( pageNum, numPages, footer, scale, extraScale, height ) {
	return page.evaluate(function(pageNum, numPages, footer, scale, extraScale, height){
		var list = footer ? document.getElementsByTagName("pd4ml:page.footer") : document.getElementsByTagName("pd4ml:page.header");
		if ( list && list[0] ) {
			var h = '<div style="border: 1 solid blue; height: ' + (height * 96) + '; width: 100%"><div style="border: 1 solid tomato; -webkit-transform-origin: 0px 0px; -webkit-transform:scale('+scale+');">' + 
			list[0].outerHTML + '</div></div>';
			h = h.replace('$[total]', numPages);
			h = h.replace('$[page]', pageNum);
			h = h.replace('$[title]', document.title);
			return h;
		} else {
			return "";
		}
		}, pageNum, numPages, footer, scale, extraScale, height);
}
